import { Params, HookContext, Id } from '@feathersjs/feathers';
import { Promisable } from 'type-fest';

declare type MethodName = "create" | "update" | "patch" | "remove";
declare type Change<T = any> = {
    before: T;
    item: T;
};
declare type ChangesById<T = any> = {
    [key: string]: Change<T>;
    [key: number]: Change<T>;
};
declare type ManipulateParams = (params: Params, context: HookContext) => (Promisable<Params>);
interface HookChangesByIdOptions {
    skipHooks: boolean;
    params?: ManipulateParams;
    deleteParams?: string[];
    name?: string | string[];
    /** @default false */
    fetchBefore?: boolean;
}
interface ViewContext<T = any> {
    item: Change<T>;
    subscription: Subscription;
    subscriptions: Subscription[];
    items: Change<T>[];
    context: HookContext;
}
declare type ActionOptions<T = any> = {
    subscription: SubscriptionResolved;
    items: Change<T>[];
    context: HookContext;
    view: Record<string, any>;
};
declare type Action<T = any> = (item: Change<T>, options: ActionOptions<T>) => (Promisable<void>);
declare type HookTriggerOptions = Subscription | Subscription[] | ((context: HookContext) => Promisable<Subscription | Subscription[]>);
declare type TransformView<T = any> = undefined | ((view: Record<string, any>, viewContext: ViewContext<T>) => Promisable<Record<string, any>>) | Record<string, any>;
declare type Condition = true | Record<string, any> | ((item: any, context: HookContext) => Promisable<boolean>);
interface Subscription {
    service?: string | string[];
    method?: string | string[];
    conditionsData?: Condition;
    conditionsResult?: Condition;
    conditionsBefore?: Condition;
    conditionsParams?: Condition;
    view?: TransformView;
    params?: ManipulateParams;
    /** @default true */
    isBlocking?: boolean;
    action?: Action;
    /** @default false */
    fetchBefore?: boolean;
    [key: string]: any;
    [key: number]: any;
}
interface SubscriptionResolved extends Subscription {
    dataResolved: boolean | Record<string, any>;
    resultResolved: boolean | Record<string, any>;
    beforeResolved: boolean | Record<string, any>;
    paramsResolved: Params;
    identifier: string;
}

declare const trigger: (options: HookTriggerOptions, action: Action) => (context: HookContext) => Promise<HookContext>;

declare const changesById: <T>(cb: (changesById: Record<Id, Change<T>>, context: HookContext) => void | Promise<void>, _options?: Partial<HookChangesByIdOptions>) => (context: HookContext) => Promise<HookContext>;

export { Action, ActionOptions, Change, ChangesById, Condition, HookChangesByIdOptions, HookTriggerOptions, ManipulateParams, MethodName, Subscription, SubscriptionResolved, TransformView, changesById, trigger };
